package com.sun.java.swing.plaf.motif;

import java.beans.PropertyVetoException;
import java.awt.event.MouseAdapter;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JButton;
import java.awt.Graphics;
import java.awt.FontMetrics;
import sun.swing.SwingUtilities2;
import sun.awt.AWTAccessor;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import javax.swing.JPanel;
import javax.swing.border.Border;
import java.awt.Dimension;
import javax.swing.JInternalFrame;
import javax.swing.JLayeredPane;
import java.awt.BorderLayout;
import javax.swing.UIManager;
import javax.swing.plaf.ComponentUI;
import javax.swing.JComponent;
import java.awt.Font;
import java.util.EventListener;
import javax.swing.JPopupMenu;
import javax.swing.Icon;
import javax.swing.plaf.basic.BasicDesktopIconUI;

public class MotifDesktopIconUI extends BasicDesktopIconUI
{
    protected MotifDesktopIconUI.DesktopIconActionListener desktopIconActionListener;
    protected MotifDesktopIconUI.DesktopIconMouseListener desktopIconMouseListener;
    protected Icon defaultIcon;
    protected MotifDesktopIconUI.IconButton iconButton;
    protected MotifDesktopIconUI.IconLabel iconLabel;
    private MotifInternalFrameTitlePane sysMenuTitlePane;
    JPopupMenu systemMenu;
    EventListener mml;
    static final int LABEL_HEIGHT = 18;
    static final int LABEL_DIVIDER = 4;
    static final Font defaultTitleFont;
    
    public static ComponentUI createUI(JComponent component) {
        return new MotifDesktopIconUI();
    }
    
    public MotifDesktopIconUI() {
    }
    
    @Override
    protected void installDefaults() {
        super.installDefaults();
        this.setDefaultIcon(UIManager.getIcon("DesktopIcon.icon"));
        this.iconButton = this.createIconButton(this.defaultIcon);
        this.sysMenuTitlePane = new MotifInternalFrameTitlePane(this.frame);
        this.systemMenu = this.sysMenuTitlePane.getSystemMenu();
        MotifBorders.FrameBorder frameBorder = new MotifBorders.FrameBorder(this.desktopIcon);
        this.desktopIcon.setLayout(new BorderLayout());
        this.iconButton.setBorder(frameBorder);
        this.desktopIcon.add(this.iconButton, "Center");
        (this.iconLabel = this.createIconLabel(this.frame)).setBorder(frameBorder);
        this.desktopIcon.add(this.iconLabel, "South");
        this.desktopIcon.setSize(this.desktopIcon.getPreferredSize());
        this.desktopIcon.validate();
        JLayeredPane.putLayer(this.desktopIcon, JLayeredPane.getLayer(this.frame));
    }
    
    @Override
    protected void installComponents() {
    }
    
    @Override
    protected void uninstallComponents() {
    }
    
    @Override
    protected void installListeners() {
        super.installListeners();
        this.desktopIconActionListener = this.createDesktopIconActionListener();
        this.desktopIconMouseListener = this.createDesktopIconMouseListener();
        this.iconButton.addActionListener(this.desktopIconActionListener);
        this.iconButton.addMouseListener(this.desktopIconMouseListener);
        this.iconLabel.addMouseListener(this.desktopIconMouseListener);
    }
    
    JInternalFrame.JDesktopIcon getDesktopIcon() {
        return this.desktopIcon;
    }
    
    void setDesktopIcon(JInternalFrame.JDesktopIcon desktopIcon) {
        this.desktopIcon = desktopIcon;
    }
    
    JInternalFrame getFrame() {
        return this.frame;
    }
    
    void setFrame(JInternalFrame frame) {
        this.frame = frame;
    }
    
    protected void showSystemMenu() {
        this.systemMenu.show(this.iconButton, 0, this.getDesktopIcon().getHeight());
    }
    
    protected void hideSystemMenu() {
        this.systemMenu.setVisible(false);
    }
    
    protected MotifDesktopIconUI.IconLabel createIconLabel(JInternalFrame internalFrame) {
        return new MotifDesktopIconUI.IconLabel(internalFrame);
    }
    
    protected MotifDesktopIconUI.IconButton createIconButton(Icon icon) {
        return new MotifDesktopIconUI.IconButton(icon);
    }
    
    protected MotifDesktopIconUI.DesktopIconActionListener createDesktopIconActionListener() {
        return new MotifDesktopIconUI.DesktopIconActionListener();
    }
    
    protected MotifDesktopIconUI.DesktopIconMouseListener createDesktopIconMouseListener() {
        return new MotifDesktopIconUI.DesktopIconMouseListener();
    }
    
    @Override
    protected void uninstallDefaults() {
        super.uninstallDefaults();
        this.desktopIcon.setLayout(null);
        this.desktopIcon.remove(this.iconButton);
        this.desktopIcon.remove(this.iconLabel);
    }
    
    @Override
    protected void uninstallListeners() {
        super.uninstallListeners();
        this.iconButton.removeActionListener(this.desktopIconActionListener);
        this.iconButton.removeMouseListener(this.desktopIconMouseListener);
        this.sysMenuTitlePane.uninstallListeners();
    }
    
    @Override
    public Dimension getMinimumSize(JComponent component) {
        JInternalFrame internalFrame = this.desktopIcon.getInternalFrame();
        int iconWidth = this.defaultIcon.getIconWidth();
        int height = this.defaultIcon.getIconHeight() + 18 + 4;
        Border border = internalFrame.getBorder();
        if (border != null) {
            iconWidth += border.getBorderInsets(internalFrame).left + border.getBorderInsets(internalFrame).right;
            height += border.getBorderInsets(internalFrame).bottom + border.getBorderInsets(internalFrame).top;
        }
        return new Dimension(iconWidth, height);
    }
    
    @Override
    public Dimension getPreferredSize(JComponent component) {
        return this.getMinimumSize(component);
    }
    
    @Override
    public Dimension getMaximumSize(JComponent component) {
        return this.getMinimumSize(component);
    }
    
    public Icon getDefaultIcon() {
        return this.defaultIcon;
    }
    
    public void setDefaultIcon(Icon defaultIcon) {
        this.defaultIcon = defaultIcon;
    }
    
    static {
        defaultTitleFont = new Font("SansSerif", 0, 12);
    }
    
    protected class IconLabel extends JPanel
    {
        JInternalFrame frame;
        
        IconLabel(JInternalFrame frame) {
            this.frame = frame;
            this.setFont(MotifDesktopIconUI.defaultTitleFont);
            this.addMouseMotionListener(new MouseMotionListener() {
                @Override
                public void mouseDragged(MouseEvent mouseEvent) {
                    MotifDesktopIconUI.IconLabel.this.forwardEventToParent(mouseEvent);
                }
                
                @Override
                public void mouseMoved(MouseEvent mouseEvent) {
                    MotifDesktopIconUI.IconLabel.this.forwardEventToParent(mouseEvent);
                }
            });
            this.addMouseListener(new MouseListener() {
                @Override
                public void mouseClicked(MouseEvent mouseEvent) {
                    MotifDesktopIconUI.IconLabel.this.forwardEventToParent(mouseEvent);
                }
                
                @Override
                public void mousePressed(MouseEvent mouseEvent) {
                    MotifDesktopIconUI.IconLabel.this.forwardEventToParent(mouseEvent);
                }
                
                @Override
                public void mouseReleased(MouseEvent mouseEvent) {
                    MotifDesktopIconUI.IconLabel.this.forwardEventToParent(mouseEvent);
                }
                
                @Override
                public void mouseEntered(MouseEvent mouseEvent) {
                    MotifDesktopIconUI.IconLabel.this.forwardEventToParent(mouseEvent);
                }
                
                @Override
                public void mouseExited(MouseEvent mouseEvent) {
                    MotifDesktopIconUI.IconLabel.this.forwardEventToParent(mouseEvent);
                }
            });
        }
        
        void forwardEventToParent(MouseEvent mouseEvent) {
            MouseEvent e = new MouseEvent(this.getParent(), mouseEvent.getID(), mouseEvent.getWhen(), mouseEvent.getModifiers(), mouseEvent.getX(), mouseEvent.getY(), mouseEvent.getXOnScreen(), mouseEvent.getYOnScreen(), mouseEvent.getClickCount(), mouseEvent.isPopupTrigger(), 0);
            AWTAccessor.MouseEventAccessor mouseEventAccessor = AWTAccessor.getMouseEventAccessor();
            mouseEventAccessor.setCausedByTouchEvent(e, mouseEventAccessor.isCausedByTouchEvent(mouseEvent));
            this.getParent().dispatchEvent(e);
        }
        
        @Override
        public boolean isFocusTraversable() {
            return false;
        }
        
        @Override
        public Dimension getMinimumSize() {
            return new Dimension(MotifDesktopIconUI.this.defaultIcon.getIconWidth() + 1, 22);
        }
        
        @Override
        public Dimension getPreferredSize() {
            String title = this.frame.getTitle();
            FontMetrics fontMetrics = this.frame.getFontMetrics(MotifDesktopIconUI.defaultTitleFont);
            int width = 4;
            if (title != null) {
                width += SwingUtilities2.stringWidth(this.frame, fontMetrics, title);
            }
            return new Dimension(width, 22);
        }
        
        @Override
        public void paint(Graphics g) {
            super.paint(g);
            int n = this.getWidth() - 1;
            g.setColor(UIManager.getColor("inactiveCaptionBorder").darker().darker());
            g.setClip(0, 0, this.getWidth(), this.getHeight());
            g.drawLine(n - 1, 1, n - 1, 1);
            g.drawLine(n, 0, n, 0);
            g.setColor(UIManager.getColor("inactiveCaption"));
            g.fillRect(2, 1, n - 3, 19);
            g.setClip(2, 1, n - 4, 18);
            int y = 18 - SwingUtilities2.getFontMetrics(this.frame, g).getDescent();
            g.setColor(UIManager.getColor("inactiveCaptionText"));
            String title = this.frame.getTitle();
            if (title != null) {
                SwingUtilities2.drawString(this.frame, g, title, 4, y);
            }
        }
    }
    
    protected class IconButton extends JButton
    {
        Icon icon;
        
        IconButton(Icon icon) {
            super(icon);
            this.icon = icon;
            this.addMouseMotionListener(new MouseMotionListener() {
                @Override
                public void mouseDragged(MouseEvent mouseEvent) {
                    MotifDesktopIconUI.IconButton.this.forwardEventToParent(mouseEvent);
                }
                
                @Override
                public void mouseMoved(MouseEvent mouseEvent) {
                    MotifDesktopIconUI.IconButton.this.forwardEventToParent(mouseEvent);
                }
            });
            this.addMouseListener(new MouseListener() {
                @Override
                public void mouseClicked(MouseEvent mouseEvent) {
                    MotifDesktopIconUI.IconButton.this.forwardEventToParent(mouseEvent);
                }
                
                @Override
                public void mousePressed(MouseEvent mouseEvent) {
                    MotifDesktopIconUI.IconButton.this.forwardEventToParent(mouseEvent);
                }
                
                @Override
                public void mouseReleased(MouseEvent mouseEvent) {
                    if (!MotifDesktopIconUI.this.systemMenu.isShowing()) {
                        MotifDesktopIconUI.IconButton.this.forwardEventToParent(mouseEvent);
                    }
                }
                
                @Override
                public void mouseEntered(MouseEvent mouseEvent) {
                    MotifDesktopIconUI.IconButton.this.forwardEventToParent(mouseEvent);
                }
                
                @Override
                public void mouseExited(MouseEvent mouseEvent) {
                    MotifDesktopIconUI.IconButton.this.forwardEventToParent(mouseEvent);
                }
            });
        }
        
        void forwardEventToParent(MouseEvent mouseEvent) {
            MouseEvent e = new MouseEvent(this.getParent(), mouseEvent.getID(), mouseEvent.getWhen(), mouseEvent.getModifiers(), mouseEvent.getX(), mouseEvent.getY(), mouseEvent.getXOnScreen(), mouseEvent.getYOnScreen(), mouseEvent.getClickCount(), mouseEvent.isPopupTrigger(), 0);
            AWTAccessor.MouseEventAccessor mouseEventAccessor = AWTAccessor.getMouseEventAccessor();
            mouseEventAccessor.setCausedByTouchEvent(e, mouseEventAccessor.isCausedByTouchEvent(mouseEvent));
            this.getParent().dispatchEvent(e);
        }
        
        @Override
        public boolean isFocusTraversable() {
            return false;
        }
    }
    
    protected class DesktopIconActionListener implements ActionListener
    {
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            MotifDesktopIconUI.this.systemMenu.show(MotifDesktopIconUI.this.iconButton, 0, MotifDesktopIconUI.this.getDesktopIcon().getHeight());
        }
    }
    
    protected class DesktopIconMouseListener extends MouseAdapter
    {
        @Override
        public void mousePressed(MouseEvent mouseEvent) {
            if (mouseEvent.getClickCount() > 1) {
                try {
                    MotifDesktopIconUI.this.getFrame().setIcon(false);
                }
                catch (PropertyVetoException ex) {}
                MotifDesktopIconUI.this.systemMenu.setVisible(false);
                MotifDesktopIconUI.this.getFrame().getDesktopPane().getDesktopManager().endDraggingFrame((JComponent)mouseEvent.getSource());
            }
        }
    }
}
